/*
  fcnc4m.h

  userspace HAL program for FCNC4M panel from Adtech.

  Copyright (C) 2019 Kjell Claesson <kjell.claesson at gmail.com>.

  Based on a work (test-modbus program, part of libmodbus) which is
  Copyright (C) 2001-2005 Stéphane Raimbault <stephane.raimbault@free.fr>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation, version 2.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301-1307  USA.

  see 'man fcnc4m'.

 */
#ifndef FCNC4M_H
#define FCNC4M_H

#ifndef ULAPI
#error This is intended as a userspace component only.
#endif

#ifdef DEBUG
#define DBG(fmt, ...)					\
        do {						\
            if (param.debug) printf(fmt,  ## __VA_ARGS__);	\
        } while(0)
#else
#define DBG(fmt, ...)
#endif

#include <stdio.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <signal.h>
#include <errno.h>
#include <getopt.h>
#include <math.h>
#include <signal.h>
#include <stdarg.h>

#include "rtapi.h"
#include "hal.h"
#include <modbus.h>

// use explicit booleans
#ifndef FALSE
typedef enum ebool { FALSE, TRUE } bool_t;
#else
typedef int bool_t;
#endif


// command registers for FCNC4M panel
#define REG_LED             0x0002  // Indication LED's on keyboard
#define BAUD_REG            0x0004  // Baud rate register
// keyboard registers for FCNC4M
#define SR_KEYBOARD         0x0001  // Register to read buttons
#define SR_KNOBS_SRE        0x0003  // Register to read Knobs (Function, Override) and E-Stop Run Stop


/* HAL data struct */
typedef struct {
    hal_bit_t	*e_stop;      // E-stop
    hal_bit_t	*stop;        // CycleStop
    hal_bit_t   *run;        // CycleRun
    hal_u32_t   *mode;
    hal_s32_t   *feed_override;
    hal_s32_t   *speed_override;
    hal_u32_t   *first_key;
    hal_u32_t   *second_key;

    hal_bit_t   *x_is_home;    // HAL Input X is homed
    hal_bit_t   *y_is_home;    // HAL Input y is homed
    hal_bit_t   *z_is_home;    // HAL Input z is homed
        
    hal_bit_t	*modbus_ok;	// the last MODBUS_OK transactions returned successfully
    hal_s32_t	*errorcount;    // number of failed Modbus transactions - hints at logical errors
    
} haldata_t;

// configuration and execution state
typedef struct params {
    char *modname;
    int modbus_debug;
    int debug;
    int slave;
    char *device;
    int baud;
    int bits;
    char parity;
    int stopbits;
    char *progname;
    int reconnect_delay;
    modbus_t *ctx;
    haldata_t *haldata;
    int hal_comp_id;
    int read_initial_done;
    int modbus_ok;
    uint16_t failed_reg;		// remember register for failed modbus transaction for debugging
    int	last_errno;
    int report_device;
    int x_home;
    int y_home;
    int z_home;
} params_type, *param_pointer;

// default options; read from command line
static params_type param = {
        .modname = "fcnc4m",
        .modbus_debug = 0,
        .debug = 0,
        .slave = 1,
        .device = "/dev/ttyS0",
        .baud = 9600,
        .parity = 'N',
        .stopbits = 1,
        .progname = "fcnc4m",
        .reconnect_delay = 1,
        .ctx = NULL,
        .haldata = NULL,
        .hal_comp_id = -1,
        .read_initial_done = 0,
        .modbus_ok = 0,    // set modbus-ok bit if last MODBUS_OK transactions went well
        .failed_reg =0,
        .last_errno = 0,
        .report_device = 0,
        .x_home = 0,
        .y_home = 0,
        .z_home = 0,
};


static int connection_state;
enum connstate {NOT_CONNECTED, OPENING, CONNECTING, CONNECTED, RECOVER, DONE};

static char *option_string = "dhm:P:";
static struct option long_options[] = {
        {"debug", no_argument, 0, 'd'},
        {"help", no_argument, 0, 'h'},
        {"modbus-debug", no_argument, 0, 'm'},
        {"port", required_argument, 0, 'P'},     // default: /dev/ttyS0
        {0,0,0,0}
};

#endif /* FCNC4M_H */
